import requests
from bs4 import BeautifulSoup
import json
import re
import warnings

class CheckLiveIG:
    def __init__(self):
        warnings.filterwarnings("ignore", category=requests.packages.urllib3.exceptions.InsecureRequestWarning)
        self.session = requests.Session()
        self.cookies = None
        self.csrf_token = None
        self.ig_app_id = None
        self.base_url = "https://www.instagram.com"
        
    def get_initial_data(self, username):
        url = f"{self.base_url}/{username}/"
        
        headers = {
            'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36",
            'Accept': "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            'sec-ch-ua': "\"Chromium\";v=\"134\", \"Not:A-Brand\";v=\"24\", \"Google Chrome\";v=\"134\"",
            'sec-ch-ua-mobile': "?0",
            'sec-ch-ua-platform': "\"Windows\"",
            'upgrade-insecure-requests': "1",
            'sec-purpose': "prefetch;prerender",
            'purpose': "prefetch",
            'sec-fetch-site': "none",
            'sec-fetch-mode': "navigate",
            'sec-fetch-user': "?1",
            'sec-fetch-dest': "document",
            'accept-language': "vi",
            'priority': "u=0, i"
        }
        
        try:
            response = self.session.get(url, headers=headers, verify=False)
            
            if response.status_code != 200:
                print(f"Failed to get page. Status code: {response.status_code}")
                print(response.text[:500])
                return False
            
            self.cookies = self.session.cookies.get_dict()
            # print("Cookies:", self.cookies)
            
            soup = BeautifulSoup(response.text, 'html.parser')
            script_tags = soup.find_all('script')
            

            for script in script_tags:
                if script.string and 'csrf_token' in script.string:
                    try:
                        match = re.search(r'"csrf_token":"(.*?)"', script.string)
                        if match:
                            self.csrf_token = match.group(1)
                            break
                    except:
                        continue

            if not self.csrf_token:
                for script in script_tags:
                    if script.string and 'window.__initialData' in script.string:
                        try:
                            json_str = script.string.split('window.__initialData = ')[1].rstrip(';')
                            data = json.loads(json_str)
                            self.csrf_token = data.get('config', {}).get('csrf_token')
                            if self.csrf_token:
                                break
                        except:
                            continue
            

            if not self.csrf_token and 'csrftoken' in self.cookies:
                self.csrf_token = self.cookies['csrftoken']
                

            for script in script_tags:
                if script.string and ('X-IG-App-ID' in script.string or 'customHeaders' in script.string):
                    try:
                        app_id_match = re.search(r'"X-IG-App-ID":"(\d+)"', script.string)
                        if app_id_match:
                            self.ig_app_id = app_id_match.group(1)
                            break
                    except:
                        continue

            if not self.ig_app_id:
                patterns = [
                    r'"X-IG-App-ID":"(\d+)"',
                    r'X-IG-App-ID":\s*"(\d+)"',
                    r'X-IG-App-ID=(\d+)',
                    r'"X-IG-App-ID"="(\d+)"'
                ]
                
                for pattern in patterns:
                    match = re.search(pattern, response.text)
                    if match:
                        self.ig_app_id = match.group(1)
                        break
            

            if not self.ig_app_id:
                self.ig_app_id = "936619743392459"
                
            # print("CSRF Token:", self.csrf_token)
            # print("X-IG-App-ID:", self.ig_app_id)

            
            return self.cookies is not None and self.csrf_token is not None and self.ig_app_id is not None
            
        except Exception as e:
            print(f"Error in get_initial_data: {e}")
            return False

    def check_profile(self, username):
   
        if not self.cookies or not self.csrf_token or not self.ig_app_id:
            print("Missing required data. Call get_initial_data first.")
            return "DIE", None
        
        url = f"{self.base_url}/api/v1/users/web_profile_info/"
        params = {
            'username': username
        }
        
        headers = {
            'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36",
            'Accept': "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            'sec-ch-ua': "\"Chromium\";v=\"134\", \"Not:A-Brand\";v=\"24\", \"Google Chrome\";v=\"134\"",
            'sec-ch-ua-mobile': "?0",
            'sec-ch-ua-platform': "\"Windows\"",
            'upgrade-insecure-requests': "1",
            'X-IG-App-ID': self.ig_app_id,
            'X-CSRFToken': self.csrf_token,
            'sec-purpose': "prefetch;prerender",
            'purpose': "prefetch",
            'sec-fetch-site': "none",
            'sec-fetch-mode': "navigate",
            'sec-fetch-user': "?1",
            'sec-fetch-dest': "document",
            'accept-language': "vi",
            'priority': "u=0, i"
        }
        
        cookie_string = "; ".join([f"{key}={value}" for key, value in self.cookies.items()])
        headers['Cookie'] = cookie_string
        
        try:
            response = requests.get(url, params=params, headers=headers, verify=False)

            user_data = None
        
            if response.status_code == 200:
                try:
                    data = response.json()
                    if 'data' in data and 'user' in data['data'] and data['data']['user'] is not None:
                        user_data = {
                            'username': data['data']['user']['username'],
                            'full_name': data['data']['user']['full_name'],
                            'id': data['data']['user']['id'],
                            'is_private': data['data']['user']['is_private']
                        }
                 
                        return "LIVE", user_data
                    else:
                        print("Invalid response format")
                        return "DIE", None
                
                except Exception as e:
                    print("Error parsing JSON response:", e)
                    print(response.text[:500])
                    return "DIE", None
                    
            elif response.status_code == 404:
                print("Profile not found!")
                return "DIE", None
                
            else:
                print(f"Error: Status code {response.status_code}")
                print(response.text[:500])
                return "DIE", None
                
        except Exception as e:
            print(f"Error in check_profile: {e}")
            return "DIE", None

    def run(self, username):
        if not self.get_initial_data(username):
            print("Failed to get initial data")
            return "DIE", None
            
        return self.check_profile(username)


if __name__ == "__main__":
    print("Chờ xử lý...")
    checker = CheckLiveIG()
    username = 'itvanhai.dev'
    status, profile_data = checker.run(username)
    
    print(f"\nProfile status: {status}")
    
    if profile_data:
        print("Profile data retrieved successfully:")
        for key, value in profile_data.items():
            print(f"{key}: {value}")
    else:
        print("No profile data available")